@@ -1,5 +1,6 @@
 
-from IPython.core.display import display, HTML
+#from IPython.core.display import display, HTML
+from IPython.display import display, HTML
 import networkx as nx
 import matplotlib.pyplot as plt
 from pyvis.physics import Physics
@@ -15,16 +16,20 @@ from keypartx.basemodes.avn_base import nlp
 
 
 ## -- avnn edges-- ##
-def avnn(comments_en,by_review = False,other_edges = True,verbose_text = False,only_otherNouns = False, only_opiAV = True, nncompound = True, opi_verbs = opi_verbs ,neg_words = neg_words, nounDEL =nounDEL, adjDEL = adjDEL,facility=False,faciNN = False, emojis = True,check_spell = True):
+def avnn(line_en,by_review = False,other_edges = True,verbose_text = False,only_otherNouns = False, only_opiAV = True, nncompound = True, opi_verbs = opi_verbs ,neg_words = neg_words, nounDEL =nounDEL, adjDEL = adjDEL,facility=False,faciNN = False, emojis = True, emojiINtext = False,  check_spell = True,back_again = True,myBA = False,myBAverb = False):
     """
-    comments_en:  list of text 
+    line_en:  string of english review 
     include_otherEdge: greedy edges of the unmatched adj, verb, noun 
     only_otherNouns: does not have adjverb in other2N_edges other than av2n, the other2N_edeges are greedy to mach the rest adj, verb, noun instead of considering match pattern
     nounDEL : nouns to be removed 
     adjDEL : adjective to be removed 
     facility: additional noun for the missing in the include_otherEdge class, e.g. I recommend! So pretty! 
-    faciNN: add facility noun to NN edges, otherwise facility only add to AV2N in the otherEdges other than facilty-N in the matched AV2N to form greedy NN 
+    faciNN: if true, add facility noun to all NN edges, otherwise facility only add to AV2N in the otherEdges other than facilty-N in the matched AV2N to form greedy NN 
     emojis: if true, it will return a list of emojis in text, otherwise dropped 
+    emojiINtext: if ture, it will tranform emojis as noun words as part of the whole text # I love 🎅 --- I love santa_claus 
+    back_again:  visit, come, go, travel, be and others verbs 
+    myBAverb: if false means all verbs other you can you own abverbs ['visit', 'come','go', 'travel', 'be'] to add 'back','again'
+    myBA: if false ['back','again']
     """
     time_start = time.time()
     all_adjV2N_edges = []
@@ -43,9 +48,99 @@ def avnn(comments_en,by_review = False,other_edges = True,verbose_text = False,o
         only_opiAV = True
         only_otherNouns =only_otherNouns
         nncompound = True"""
-        
+    
+    sent_i = 0 
+
+    try:
+        line = line_en.replace('\n','') # remove new line in the paragraph
+        line = line.replace('\t','')
+        line1 = re.sub('[:@#$+=%*`)(><]', ' , ', line)
+        #line1 = re.sub(r'\b\d+\b', ' ', line1)
+        line1 = re.sub(" \d+", "  ", line1)
+        time.sleep(.5)
+        line1 = re.sub(r'\b' + "th" + r'\b', ' ', line1) 
+        line1 = line1.replace("."," . ") # in case: it is great.the food become a word of great.the
+        line1 = line1.replace("!"," . ") # in case: it is great.the food become a word of great.the
+        line1 = line1.replace("?"," . ") # ?,!, . for the sentence dependency 
+        #line1 = line1.replace("’","'")
+        line1 = re.sub(u"(\u2018|\u2019)", "'", line1)
+        line1 = re.sub(' +', ' ', line1).strip() #remove extra space whitespace 
+        print('line:', line1)
+        if by_review:
+            all_adjV2N_edges0, allNNedges0, all_new_nodes0, all_old_nodes0,new_node_nouns0,new_node_verbs0,new_node_adjs0,emojis0 = text2edges(line1,verbose= False, verbose_text = verbose_text,include_otherEdge = other_edges,only_otherNouns =only_otherNouns,only_opiAV = only_opiAV,nncompound= nncompound, opi_verbs = opi_verbs ,neg_words = neg_words, nounDEL =nounDEL, adjDEL = adjDEL,facility=facility,faciNN=faciNN,emojis = emojis,  emojiINtext = emojiINtext,   check_spell = check_spell,back_again = back_again,myBA = myBA,myBAverb = myBAverb)
+            #all_adjV2N_edges, allNNedges, all_adjV2N_edges_drop, all_new_nodes, all_old_nodes,new_node_nouns,new_node_verbs,new_node_adjs
+            allNNedges.extend(allNNedges0)
+            all_adjV2N_edges.extend(all_adjV2N_edges0)
+            all_new_nodes.extend(all_new_nodes0)
+            all_old_nodes.extend(all_old_nodes0)
+            new_node_nouns.extend(new_node_nouns0)
+            new_node_verbs.extend(new_node_verbs0)
+            new_node_adjs.extend(new_node_adjs0)
+            emojis1.extend(emojis0)
+        else:
+            for ib,sent in enumerate(nlp(line1).sents):   # dependency parse https://spacy.io/usage/linguistic-features#sbd sentence
+                #print(sent)
+                
+                if len(sent.text.split())>2: # minimal 2 words
+                    sent_i += 1
+                    print('sentence' + str(ib),sent.text)
+                    all_adjV2N_edges0, allNNedges0, all_new_nodes0, all_old_nodes0,new_node_nouns0,new_node_verbs0,new_node_adjs0,emojis0 = text2edges(sent.text,verbose_text = verbose_text,verbose = False,include_otherEdge = other_edges,only_otherNouns =only_otherNouns,only_opiAV = only_opiAV,nncompound= nncompound, opi_verbs = opi_verbs ,neg_words = neg_words, nounDEL =nounDEL, adjDEL = adjDEL,facility=facility,faciNN=faciNN,emojis = emojis, emojiINtext = emojiINtext,  check_spell = check_spell,back_again = back_again,myBA = myBA,myBAverb = myBAverb)
+                    
+                    allNNedges.extend(allNNedges0)
+                    all_adjV2N_edges.extend(all_adjV2N_edges0)
+                    all_new_nodes.extend(all_new_nodes0)
+                    all_old_nodes.extend(all_old_nodes0)
+                    new_node_nouns.extend(new_node_nouns0)
+                    new_node_verbs.extend(new_node_verbs0)
+                    new_node_adjs.extend(new_node_adjs0)
+                    emojis1.extend(emojis0)
+    except:
+        print('---sth wrong ---')
+    
+    print('processing time:', time.time()-time_start)
+    print('length of sentences:',sent_i)
+    return all_adjV2N_edges,allNNedges,all_new_nodes,all_old_nodes,new_node_nouns,new_node_verbs,new_node_adjs,emojis1,sent_i
+
+
+  
+# comments_en list 
+def avnn2(comments_en,by_review = False,other_edges = True,verbose_text = False,only_otherNouns = False, only_opiAV = True, nncompound = True, opi_verbs = opi_verbs ,neg_words = neg_words, nounDEL =nounDEL, adjDEL = adjDEL,facility=False,faciNN = False, emojis = True, emojiINtext = False, check_spell = True,back_again = True,myBA = False,myBAverb = False):
+    """
+    comments_en:  list of text 
+    include_otherEdge: greedy edges of the unmatched adj, verb, noun 
+    only_otherNouns: does not have adjverb in other2N_edges other than av2n, the other2N_edeges are greedy to mach the rest adj, verb, noun instead of considering match pattern
+    nounDEL : nouns to be removed 
+    adjDEL : adjective to be removed 
+    facility: additional noun for the missing in the include_otherEdge class, e.g. I recommend! So pretty! 
+    faciNN: add facility noun to NN edges, otherwise facility only add to AV2N in the otherEdges other than facilty-N in the matched AV2N to form greedy NN 
+    emojis: if true, it will return a list of emojis in text, otherwise dropped 
+    emojiINtext: if ture, it will tranform emojis as noun words as part of the whole text # I love 🎅 --- I love santa_claus 
+    back_again:  visit, come, go, travel, be and others verbs 
+    myBAverb: if false means all verbs other you can you own abverbs ['visit', 'come','go', 'travel', 'be'] to add 'back','again'
+    myBA: if false ['back','again']
+    """
+    time_start = time.time()
+    all_adjV2N_edges = []
+    allNNedges = [] # noun to noun is the noun in adjV2N 
+    all_new_nodes= []
+    all_old_nodes= []
+    new_node_nouns= []
+    new_node_verbs= []
+    new_node_adjs= []
+    emojis1 = []
+
+    """    by_review = False # by review or by sentence in review
+        #opinion_verbs = False
+        other_edges = True # not match verb, adj, noun
+        verbose_text = False
+        only_opiAV = True
+        only_otherNouns =only_otherNouns
+        nncompound = True"""
+    line_i = 0   
+    sent_i = 0 
     for ia,line_ori in enumerate(comments_en):
         print(ia)
+        line_i +=1
         try:
             line = line_ori.replace('\n','') # remove new line in the paragraph
             line = line.replace('\t','')
@@ -62,7 +157,7 @@ def avnn(comments_en,by_review = False,other_edges = True,verbose_text = False,o
             line1 = re.sub(' +', ' ', line1).strip() #remove extra space whitespace 
             print('line:', line1)
             if by_review:
-                all_adjV2N_edges0, allNNedges0, all_new_nodes0, all_old_nodes0,new_node_nouns0,new_node_verbs0,new_node_adjs0,emojis0 = text2edges(line1,verbose= False, verbose_text = verbose_text,include_otherEdge = other_edges,only_otherNouns =only_otherNouns,only_opiAV = only_opiAV,nncompound= nncompound, opi_verbs = opi_verbs ,neg_words = neg_words, nounDEL =nounDEL, adjDEL = adjDEL,facility=facility,faciNN=faciNN,emojis = emojis,check_spell = check_spell)
+                all_adjV2N_edges0, allNNedges0, all_new_nodes0, all_old_nodes0,new_node_nouns0,new_node_verbs0,new_node_adjs0,emojis0 = text2edges(line1,verbose= False, verbose_text = verbose_text,include_otherEdge = other_edges,only_otherNouns =only_otherNouns,only_opiAV = only_opiAV,nncompound= nncompound, opi_verbs = opi_verbs ,neg_words = neg_words, nounDEL =nounDEL, adjDEL = adjDEL,facility=facility,faciNN=faciNN,emojis = emojis, emojiINtext = emojiINtext, check_spell = check_spell,back_again = back_again,myBA = myBA,myBAverb = myBAverb)
                 #all_adjV2N_edges, allNNedges, all_adjV2N_edges_drop, all_new_nodes, all_old_nodes,new_node_nouns,new_node_verbs,new_node_adjs
                 allNNedges.extend(allNNedges0)
                 all_adjV2N_edges.extend(all_adjV2N_edges0)
@@ -75,9 +170,11 @@ def avnn(comments_en,by_review = False,other_edges = True,verbose_text = False,o
             else:
                 for ib,sent in enumerate(nlp(line1).sents):   # dependency parse https://spacy.io/usage/linguistic-features#sbd sentence
                     #print(sent)
+                    
                     if len(sent.text.split())>2: # minimal 2 words
+                        sent_i += 1
                         print('sentence' + str(ib),sent.text)
-                        all_adjV2N_edges0, allNNedges0, all_new_nodes0, all_old_nodes0,new_node_nouns0,new_node_verbs0,new_node_adjs0,emojis0 = text2edges(sent.text,verbose_text = verbose_text,verbose = False,include_otherEdge = other_edges,only_otherNouns =only_otherNouns,only_opiAV = only_opiAV,nncompound= nncompound, opi_verbs = opi_verbs ,neg_words = neg_words, nounDEL =nounDEL, adjDEL = adjDEL,facility=facility,faciNN=faciNN,emojis = emojis,check_spell = check_spell)
+                        all_adjV2N_edges0, allNNedges0, all_new_nodes0, all_old_nodes0,new_node_nouns0,new_node_verbs0,new_node_adjs0,emojis0 = text2edges(sent.text,verbose_text = verbose_text,verbose = False,include_otherEdge = other_edges,only_otherNouns =only_otherNouns,only_opiAV = only_opiAV,nncompound= nncompound, opi_verbs = opi_verbs ,neg_words = neg_words, nounDEL =nounDEL, adjDEL = adjDEL,facility=facility,faciNN=faciNN,emojis = emojis,emojiINtext = emojiINtext,  check_spell = check_spell,back_again = back_again,myBA = myBA,myBAverb = myBAverb)
                         
                         allNNedges.extend(allNNedges0)
                         all_adjV2N_edges.extend(all_adjV2N_edges0)
@@ -91,6 +188,8 @@ def avnn(comments_en,by_review = False,other_edges = True,verbose_text = False,o
             print('---sth wrong ---')
         
     print('processing time:', time.time()-time_start)
+    print('length of reviews:',line_i)
+    print('length of sentences:',sent_i)
     return all_adjV2N_edges,allNNedges,all_new_nodes,all_old_nodes,new_node_nouns,new_node_verbs,new_node_adjs,emojis1
 
 
@@ -105,7 +204,8 @@ def keynet(all_edges1,heading='avN_N Network',core_K =1,plot_graph = True,save_a
     Gd = nx.DiGraph() 
     #all_edges1 = all_adjV2N_edges_w + allNNEdges_dir_w
     Gd.add_weighted_edges_from(all_edges1)
-    print('length Gd nodes: ',len(Gd.nodes))
+    #print('length Gd nodes: ',len(Gd.nodes))
+    print('length Gd edges: ',len(all_edges1))
     #print(Gd.nodes)
 
 
@@ -165,10 +265,10 @@ def keynet(all_edges1,heading='avN_N Network',core_K =1,plot_graph = True,save_a
     print('length Gd nodes after error node removed:',len(Gd.nodes))
     print('all edges after k-core: ', len(all_edges))
     
+    nt.show_buttons(filter_=['physics']) 
     if plot_graph:
-        nt.show_buttons(filter_=['physics']) 
-        nt.show("network_avnn.html")
-        display(HTML('network_avnn.html'))
+        display(nt.show("network_avnn.html"))
+        #display(HTML('network_avnn.html'))
     if save_add:      
         nt.save_graph(save_add)
     # all_edges after k-core of all_edges1
@@ -185,9 +285,9 @@ def communet(partition,iGd,Gdc,all_edges,colorList_rgba1,plot_graph = True,save_
     #from itertools import cycle, islice
     colorList1  = list(islice(cycle(colorList_rgba1),len(partition)))
 
-    ntc=Network(height=height, width=width,heading=heading,bgcolor='white',font_color="black", directed = directed, notebook =notebook ,cdn=cdn)
+    ntc=Network(height=height, width=width,heading=heading,bgcolor='white',font_color="black", directed = directed, notebook =notebook ,cdn_resources=cdn)
     ntc.set_edge_smooth('cubicBezier')
-    error_noeds =[]
+    #error_noeds =[]
     for com, color in zip(partition, colorList1):
       for index in com:
         
@@ -225,10 +325,11 @@ def communet(partition,iGd,Gdc,all_edges,colorList_rgba1,plot_graph = True,save_
       edge1 = re.sub(r'[2][a-zA-Z]{2}',"2",edge[1])
       ntc.add_edge(edge0,edge1, title = elabel,width = value)
       #ntc.add_edge(edge[0][:-2],edge[1][:-2], title = elabel,width = value)
+    ntc.show_buttons(filter_=['physics']) 
     if plot_graph:
-        ntc.show_buttons(filter_=['physics'])  
-        ntc.show('community_network.html')
-        display(HTML('community_network.html'))
+         
+        display(ntc.show('community_network.html'))
+        #display(HTML('community_network.html'))
     if save_add:
         ntc.save_graph(save_add)
         
@@ -278,7 +379,7 @@ def communet_nnc(partition,iGd,Gdc,all_edges,colorList_rgba1,plot_graph = True,s
     ## nn com 
     colorList1  = list(islice(cycle(colorList_rgba1),len(nodes_NN_index )))
 
-    ntc2 =Network(height=height, width=width,heading=heading,bgcolor=bgcolor,font_color=font_color, directed =directed, notebook =notebook,cdn=cdn)
+    ntc2 =Network(height=height, width=width,heading=heading,bgcolor=bgcolor,font_color=font_color, directed =directed, notebook =notebook,cdn_resources=cdn)
     ntc2.set_edge_smooth('cubicBezier')
     com_nn_a = 0
     com_nn_aa = []
@@ -341,11 +442,10 @@ def communet_nnc(partition,iGd,Gdc,all_edges,colorList_rgba1,plot_graph = True,s
       ntc2.add_edge(edge0,edge1, title = elabel,width = value)
 
       #ntc2.add_edge(edge[0][:-2],edge[1][:-2], title = elabel,width = value)
-
+    ntc2.show_buttons(filter_=['physics']) 
     if plot_graph:
-        ntc2.show_buttons(filter_=['physics'])  
-        ntc2.show('communityNNcolor.html')
-        display(HTML('communityNNcolor.html'))
+        display(ntc2.show('communityNNcolor.html'))
+        #display(HTML('communityNNcolor.html'))
     if save_add:
         ntc2.save_graph(save_add)
 
@@ -394,7 +494,7 @@ def gray_unit(nnColor_df,Gdc,all_edges,comLen = False,plot_graph = False, save_f
       #print(len(set(comEdges)))
       comEdges = sorted(set(comEdges))
 
-      ntc3=Network(height=height, width=width,heading=heading,bgcolor=bgcolor,font_color=font_color, directed = directed, notebook =notebook,cdn=cdn)
+      ntc3=Network(height=height, width=width,heading=heading,bgcolor=bgcolor,font_color=font_color, directed = directed, notebook =notebook,cdn_resources=cdn)
       ntc3.set_edge_smooth('cubicBezier')
 
 
@@ -445,11 +545,11 @@ def gray_unit(nnColor_df,Gdc,all_edges,comLen = False,plot_graph = False, save_f
             edge1 = re.sub(r'[2][a-zA-Z]{2}',"2",edge[1])
             ntc3.add_edge(edge0,edge1, title = elabel,width = value)
             #ntc3.add_edge(edge[0][:-2],edge[1][:-2], title = elabel,width = value)
+      ntc3.show_buttons(filter_=['physics'])
       if plot_graph:
-        ntc3.show_buttons(filter_=['physics']) 
         if community_index == com_len-1: 
-            ntc3.show(str(community_index) + '_gray_unit.html')
-            display(HTML(str(community_index) + '_gray_unit.html'))
+            display(ntc3.show(str(community_index) + '_gray_unit.html'))
+            #display(HTML(str(community_index) + '_gray_unit.html'))
       if save_folder:
         ntc3.save_graph(save_folder+ '/{}.html'.format(community_index +1))
 
@@ -495,4 +595,125 @@ def gray_unit(nnColor_df,Gdc,all_edges,comLen = False,plot_graph = False, save_f
     return colorGrayDF
   
 
-#comments_en =["Once again a memorable visit to Arctic Light. Hearty staff, tasty breakfast (special mention of crispy waffles and rice porridge!), Clean and well-equipped room. Good bed. High quality and tastefully decorated. ❤️ We recommend and book your next stay at the Arctic Light Hotel. 😊"]
+def compare_net(allWedges, weight_K=1, specEdges1 = False,specEdges2 = False, specNodes1=False,specNodes2=False, color1= 'red', color2 = 'green', save_add = False,heading = 'Compared Network',core_K = 1 , plot_graph = True, height='600px',width='100%',bgcolor='white', font_color="black",directed = True,notebook =True,cdn='remote'):
+    """ 
+    allWEdges  of list of weighted edges including SharedA, ShareB, SpecificA, SpecificB edges 
+    if specedge or specnodes False, color1 , color2 do not work all show gray color
+    
+    """
+    nt=Network(heading=heading, height=height, width=width,bgcolor=bgcolor,font_color=font_color, directed = directed, notebook =notebook,cdn_resources=cdn)
+    nt.set_edge_smooth('cubicBezier')
+    allWedges1 = []
+    for wedge in allWedges:
+        if wedge[2]>= weight_K:
+            allWedges1.append(wedge)
+    
+    Gd = nx.DiGraph() 
+    Gd.add_weighted_edges_from(allWedges1)
+    #print('length Gd edges: ',len( allWedge))
+
+    error_noeds =[]
+    all_nodes = []
+    specNodes1_new = []
+    specNodes2_new = []
+    values = []
+    for node,degree in dict(Gd.degree).items():
+      #degree = math.log(degree,100)
+      try:
+          if node in specNodes1:
+                nodeColor = color1
+                specNodes1_new.append((node,degree))
+          elif node in specNodes2:
+             nodeColor = color2
+             specNodes2_new.append((node,degree))
+          else:
+            nodeColor = 'gray'
+      except:
+        nodeColor = "gray"
+
+
+
+      degree1 = 0.01*degree
+      color =""
+      shape =""
+      node1 = ""
+      if 'vvv' in node:
+        color = nodeColor
+        shape = 'square'   # shape  image, circularImage, diamond, dot, star, triangle, triangleDown, square and icon.
+        #node1 = node[:-2]
+        node1 = re.sub(r'[2][a-zA-Z]{2}',"2",node)
+      elif 'aaa' in node:
+        color = nodeColor
+        shape = 'triangle'
+        #node1 = node[:-2]
+        node1 = re.sub(r'[2][a-zA-Z]{2}',"2",node)
+      elif 'nnn' in node:
+        color = nodeColor
+        shape = 'dot'
+        #node1 = node[:-2]
+        node1 = re.sub(r'[2][a-zA-Z]{2}',"2",node)
+      else:
+        error_node = node
+        #print(node)
+        Gd.remove_node(error_node)  # remove error node
+        error_noeds.append(error_node)
+      all_nodes.append((node,degree))
+      values.append(degree)
+      nt.add_node(node1,title = node[:-4].replace('2nnn'," ") +':'+str(degree),value=degree1,color = color, shape =shape) #value=degree
+    #nt.add_nodes(nodes, value = values)
+    
+    try:
+        specEdges1 = [x[:2] for x in specEdges1]
+        specEdges2 =  [x[:2] for x in specEdges2]  # edge include weight but has different size of same edge 
+
+    except:
+        print('no specific edges')
+    
+    
+    weights = []
+    all_edges =[]              # get final all_edges after errors 
+    specEdges1new = []
+    specEdges2new = []
+    
+    for edge in allWedges1:
+      try:
+          if edge[:2] in specEdges1:
+            edgeColor = color1
+            specEdges1new.append(edge)
+          elif edge[:2] in specEdges2:
+            edgeColor = color2
+            specEdges2new.append(edge)
+          else:
+            edgeColor = 'gray'
+      except: 
+         edgeColor = 'gray'
+         #print('except edge', edge)
+        
+      if any(ern in edge for ern in error_noeds): 
+        print(edge,':error edge removed')
+
+      elif all([dict(Gd.degree)[node]>=core_K for node in edge[:2]]):         # core_k if node degree is less than k then edge droped         
+        elabel = str(edge[2])
+        value = int(edge[2])
+        weights.append(edge[2])
+        edge0 = re.sub(r'[2][a-zA-Z]{2}',"2",edge[0])
+        edge1 = re.sub(r'[2][a-zA-Z]{2}',"2",edge[1])
+        nt.add_edge(edge0,edge1, title = elabel,width = value, color = edgeColor)
+        #nt.add_edge(edge[0][:-2],edge[1][:-2], title = elabel,width = value)
+        all_edges.append(edge)
+
+    nt.show_buttons(filter_=['physics']) 
+    if save_add:
+        nt.save_graph(save_add) 
+    if plot_graph:
+        display(nt.show("compare_avnn.html"))
+
+    return all_nodes, specNodes1_new,specNodes2_new ,all_edges,  specEdges1new,specEdges2new
+
+  
+  
+  
+if __name__ == "__main__":
+  comments_en =["Once again a memorable visit to Arctic Light. Hearty staff, tasty breakfast (special mention of crispy waffles and rice porridge!), Clean and well-equipped room. Good bed. High quality and tastefully decorated. ❤️ We recommend and book your next stay at the Arctic Light Hotel. 😊"]
+  comments_en = ["I love you. you are mine friend. I dont know"," you are right. I no no "]
+  av2n, nn,new_nodes,old_nodes,nodesN,nodesV,nodesA,emojis = avnn(comments_en,by_review= False,verbose_text= True, emojis= True, check_spell= False)
\ No newline at end of file
